package kr.co.vividnext.sodalive.admin.content.banner

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBanner
import kr.co.vividnext.sodalive.content.main.banner.QAudioContentBanner.audioContentBanner
import kr.co.vividnext.sodalive.event.QEvent.event
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface AdminContentBannerRepository : JpaRepository<AudioContentBanner, Long>, AdminContentBannerQueryRepository

interface AdminContentBannerQueryRepository {
    fun getAudioContentMainBannerList(): List<GetAdminContentBannerResponse>
}

class AdminContentBannerQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,
    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) : AdminContentBannerQueryRepository {
    override fun getAudioContentMainBannerList(): List<GetAdminContentBannerResponse> {
        return queryFactory
            .select(
                QGetAdminContentBannerResponse(
                    audioContentBanner.id,
                    audioContentBanner.type,
                    audioContentBanner.thumbnailImage.prepend("/").prepend(cloudFrontHost),
                    audioContentBanner.event.id,
                    audioContentBanner.event.thumbnailImage,
                    audioContentBanner.creator.id,
                    audioContentBanner.creator.nickname,
                    audioContentBanner.link,
                    audioContentBanner.isAdult
                )
            )
            .from(audioContentBanner)
            .leftJoin(audioContentBanner.event, event)
            .leftJoin(audioContentBanner.creator, member)
            .where(audioContentBanner.isActive.isTrue)
            .orderBy(audioContentBanner.orders.asc())
            .fetch()
    }
}
