package kr.co.vividnext.sodalive.admin.content.curation

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCuration
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdminContentCurationService(
    private val repository: AdminContentCurationRepository
) {
    @Transactional
    fun createContentCuration(request: CreateContentCurationRequest) {
        repository.save(
            AudioContentCuration(
                title = request.title,
                description = request.description,
                isAdult = request.isAdult
            )
        )
    }

    @Transactional
    fun updateContentCuration(request: UpdateContentCurationRequest) {
        val audioContentCuration = repository.findByIdOrNull(id = request.id)
            ?: throw SodaException("잘못된 요청입니다.")

        if (request.title != null) {
            audioContentCuration.title = request.title
        }

        if (request.description != null) {
            audioContentCuration.description = request.description
        }

        if (request.isAdult != null) {
            audioContentCuration.isAdult = request.isAdult
        }

        if (request.isActive != null) {
            audioContentCuration.isActive = request.isActive
        }
    }

    @Transactional
    fun updateContentCurationOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val audioContentCuration = repository.findByIdOrNull(ids[index])

            if (audioContentCuration != null) {
                audioContentCuration.orders = index + 1
            }
        }
    }

    fun getContentCurationList(): List<GetAdminContentCurationResponse> {
        return repository.getAudioContentCurationList()
    }
}
