package kr.co.vividnext.sodalive.admin.content.theme

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.theme.AudioContentTheme
import kr.co.vividnext.sodalive.content.theme.GetAudioContentThemeResponse
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import kr.co.vividnext.sodalive.content.theme.QGetAudioContentThemeResponse
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface AdminContentThemeRepository : JpaRepository<AudioContentTheme, Long>, AdminContentThemeQueryRepository

interface AdminContentThemeQueryRepository {
    fun findIdByTheme(theme: String): Long?
    fun getActiveThemes(): List<GetAudioContentThemeResponse>
}

class AdminContentThemeQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) : AdminContentThemeQueryRepository {
    override fun findIdByTheme(theme: String): Long? {
        return queryFactory
            .select(audioContentTheme.id)
            .from(audioContentTheme)
            .where(audioContentTheme.theme.eq(theme))
            .fetchOne()
    }

    override fun getActiveThemes(): List<GetAudioContentThemeResponse> {
        return queryFactory
            .select(
                QGetAudioContentThemeResponse(
                    audioContentTheme.id,
                    audioContentTheme.theme,
                    audioContentTheme.image.prepend("/").prepend(cloudFrontHost)
                )
            )
            .from(audioContentTheme)
            .where(audioContentTheme.isActive.isTrue)
            .orderBy(audioContentTheme.orders.asc())
            .fetch()
    }
}
