package kr.co.vividnext.sodalive.admin.explorer

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/admin/explorer")
@PreAuthorize("hasRole('ADMIN')")
class AdminExplorerController(private val service: AdminExplorerService) {
    @PostMapping
    fun createExplorerSection(@RequestBody request: CreateExplorerSectionRequest): ApiResponse<Any> {
        service.createExplorerSection(request)
        return ApiResponse.ok(null, "등록되었습니다.")
    }

    @PutMapping
    fun updateExplorerSection(@RequestBody request: UpdateExplorerSectionRequest) = ApiResponse.ok(
        service.updateExplorerSection(request),
        "수정되었습니다."
    )

    @PutMapping("/orders")
    fun updateExplorerSectionOrders(
        @RequestBody request: UpdateExplorerSectionOrdersRequest
    ) = ApiResponse.ok(
        service.updateExplorerSectionOrders(request.firstOrders, request.ids),
        "수정되었습니다."
    )

    @GetMapping
    fun getExplorerSections(pageable: Pageable) = ApiResponse.ok(service.getExplorerSections(pageable))
}
