package kr.co.vividnext.sodalive.aws.s3

import com.amazonaws.services.s3.AmazonS3Client
import com.amazonaws.services.s3.model.ObjectMetadata
import com.amazonaws.services.s3.model.PutObjectRequest
import org.slf4j.LoggerFactory
import org.springframework.stereotype.Component
import java.io.InputStream

@Component
class S3Uploader(private val amazonS3Client: AmazonS3Client) {
    private val logger = LoggerFactory.getLogger(this::class.java)

    fun upload(
        inputStream: InputStream,
        bucket: String,
        filePath: String,
        metadata: ObjectMetadata? = null
    ): String {
        putS3(inputStream, bucket, filePath, metadata)
        return filePath
    }

    private fun putS3(
        inputStream: InputStream,
        bucket: String,
        filePath: String,
        metadata: ObjectMetadata?
    ): String {
        amazonS3Client.putObject(PutObjectRequest(bucket, filePath, inputStream, metadata))
        logger.info("파일이 업로드 되었습니다.")
        return amazonS3Client.getUrl(bucket, filePath).toString()
    }
}
