package kr.co.vividnext.sodalive.faq

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.faq.QFaq.faq
import kr.co.vividnext.sodalive.faq.QFaqCategory.faqCategory
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface FaqRepository : JpaRepository<Faq, Long>

@Repository
class FaqQueryRepository(private val queryFactory: JPAQueryFactory) {
    fun getCategory(category: String): FaqCategory? {
        return queryFactory
            .selectFrom(faqCategory)
            .where(
                faqCategory.isActive.isTrue
                    .and(faqCategory.category.eq(category))
            )
            .fetchFirst()
    }

    fun getCategoryList(): List<String> {
        return queryFactory
            .select(faqCategory.category)
            .from(faqCategory)
            .where(faqCategory.isActive.isTrue)
            .orderBy(faqCategory.orders.desc())
            .fetch()
    }

    fun getFaqList(category: String): List<GetFaqResponseItem> {
        return queryFactory
            .select(
                QGetFaqResponseItem(
                    faq.id,
                    Expressions.asString(category),
                    faq.question,
                    faq.answer
                )
            )
            .from(faq)
            .innerJoin(faq.category, faqCategory)
            .where(
                faq.isActive.isTrue
                    .and(faqCategory.isActive.isTrue)
                    .and(faqCategory.category.eq(category))
            )
            .orderBy(faq.orders.desc())
            .fetch()
    }

    fun getFaq(id: Long): Faq? {
        return queryFactory
            .selectFrom(faq)
            .where(
                faq.isActive.isTrue
                    .and(faq.id.eq(id))
            )
            .fetchFirst()
    }
}
