package kr.co.vividnext.sodalive.member.auth

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/auth")
class AuthController(private val service: AuthService) {
    @PostMapping
    fun authVerify(
        @RequestBody request: AuthVerifyRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        val authenticateData = service.certificate(request, memberId = member.id!!)

        if (service.isBlockAuth(authenticateData)) {
            service.signOut(member.id!!)
            throw SodaException("운영정책을 위반하여 이용을 제한합니다.")
        } else {
            service.authenticate(authenticateData, member.id!!)
        }

        ApiResponse.ok(null, null)
    }
}
