package kr.co.vividnext.sodalive.menu

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.menu.QMenu.menu
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface MenuRepository : JpaRepository<Menu, Long>, MenuQueryRepository

interface MenuQueryRepository {
    fun getMenu(role: MemberRole): List<GetMenuResponse>
}

class MenuQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : MenuQueryRepository {
    override fun getMenu(role: MemberRole): List<GetMenuResponse> {
        return queryFactory
            .selectFrom(menu)
            .where(
                menu.isActive.isTrue
                    .and(menu.roles.eq(role))
                    .and(
                        (menu.route.isNotNull.and(menu.parent.isNull))
                            .or(menu.route.isNull.and(menu.children.isNotEmpty))
                    )
            )
            .orderBy(menu.orders.asc())
            .fetch()
            .asSequence()
            .map {
                GetMenuResponse(
                    title = it.title,
                    route = it.route,
                    items = if (it.children.isNotEmpty()) {
                        it.children
                            .asSequence()
                            .map { menu -> GetMenuResponse(menu.title, menu.route) }
                            .toList()
                    } else {
                        null
                    }
                )
            }
            .toList()
    }
}
